"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServerless = void 0;
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/no-var-requires */
const serverless_1 = tslib_1.__importDefault(require("serverless"));
const resolve_input_1 = tslib_1.__importDefault(require("serverless/lib/cli/resolve-input"));
const resolve_1 = tslib_1.__importDefault(require("serverless/lib/configuration/variables/resolve"));
const resolve_meta_1 = tslib_1.__importDefault(require("serverless/lib/configuration/variables/resolve-meta"));
const getConfiguration_1 = require("./getConfiguration");
const DEFAULT_STAGE = process.env.STAGE;
async function createServerless({ stage = DEFAULT_STAGE } = {}) {
    const { configurationPath, configuration } = await (0, getConfiguration_1.getConfiguration)();
    const { commands, options, commandSchema } = (0, resolve_input_1.default)();
    const serverless = new serverless_1.default({
        configuration,
        configurationPath,
        commands,
        options: stage ? { ...options, stage } : options,
        stage,
    });
    await serverless.init();
    await setup(serverless, configuration, options, commandSchema);
    return [serverless, configuration];
}
exports.createServerless = createServerless;
async function setup(serverless, config, options, commandSchema) {
    if (serverless.configurationInput)
        serverless.service.reloadServiceFileParam();
    // Some plugins acccess `options` through `serverless.variables`
    serverless.variables.options = serverless.pluginManager.cliOptions;
    // merge arrays after variables have been populated
    // (https://github.com/serverless/serverless/issues/3511)
    serverless.service.mergeArrays();
    // populate function names after variables are loaded in case functions were externalized
    // (https://github.com/serverless/serverless/issues/2997)
    serverless.service.setFunctionNames(serverless.processedInput.options);
    // If in context of service, validate the service configuration
    if (serverless.serviceDir)
        await serverless.service.validate();
    serverless.serviceOutputs = new Map();
    serverless.servicePluginOutputs = new Map();
    const meta = await (0, resolve_meta_1.default)(config);
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    const providerName = require('serverless/lib/configuration/resolve-provider-name')(config);
    const resolverConfiguration = {
        serviceDir: process.cwd(),
        configuration: config,
        variablesMeta: meta,
        sources: {
            env: require('serverless/lib/configuration/variables/sources/env'),
            file: require('serverless/lib/configuration/variables/sources/file'),
            opt: require('serverless/lib/configuration/variables/sources/opt'),
            self: require('serverless/lib/configuration/variables/sources/self'),
            strToBool: require('serverless/lib/configuration/variables/sources/str-to-bool'),
            sls: require('serverless/lib/configuration/variables/sources/instance-dependent/get-sls')(),
        },
        options: require('serverless/lib/cli/filter-supported-options')(options, { commandSchema, providerName }),
        fulfilledSources: new Set(),
        propertyPathsToResolve: new Set(createPathToResolve(config)),
        variableSourcesInConfig: new Set(),
    };
    await (0, resolve_1.default)(resolverConfiguration);
}
function createPathToResolve(config) {
    return Object.entries(config).flatMap(([k, v]) => v != null && !Array.isArray(v) && typeof v === 'object'
        ? Object.entries(v).flatMap(([k2, v2]) => [
            `${k}\0${k2}`,
            ...createPathToResolve(v2).map((k3) => `${k}\0${k2}\0${k3}`),
        ])
        : [k]);
}
//# sourceMappingURL=createServerless.js.map