"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runWithEnvVars = void 0;
const tslib_1 = require("tslib");
const chokidar_1 = tslib_1.__importDefault(require("chokidar"));
const path_1 = require("path");
const resolve_configuration_path_1 = tslib_1.__importDefault(require("serverless/lib/cli/resolve-configuration-path"));
const getEnvVars_1 = require("./getEnvVars");
const runChildProcess_1 = require("./runChildProcess");
const setEnvVars_1 = require("./setEnvVars");
async function runWithEnvVars(options) {
    var _a;
    const configPath = (_a = (await (0, resolve_configuration_path_1.default)())) !== null && _a !== void 0 ? _a : (0, path_1.join)(process.cwd(), 'serverless.yml');
    let cp = await createCp(options);
    if (options.watch) {
        const watcher = chokidar_1.default.watch(configPath, { persistent: true, interval: 1000 });
        watcher.on('change', async () => {
            console.log('Configuration changed, restarting...');
            cp.kill();
            cp = await createCp(options);
        });
    }
}
exports.runWithEnvVars = runWithEnvVars;
async function createCp(options) {
    const { cmd, args, log = 'off', stage, watch = false, namespace } = options;
    console.info('Reading Serverless environment variables...');
    const envVars = await (0, getEnvVars_1.getEnvVars)({ stage });
    logEnvVars(log, envVars, process.env, namespace);
    (0, setEnvVars_1.setEnvVars)(envVars, options.namespace);
    console.info('Serverless environment variables set');
    return (0, runChildProcess_1.runChildProcess)(cmd, args, process.env, watch);
}
function logEnvVars(level, sls, env, namespace) {
    if (level === 'off') {
        return;
    }
    if (level === 'sls') {
        logSlsVars(sls, env);
    }
    if (level === 'all') {
        logAllVars({ ...sls, ...env });
    }
    if (namespace) {
        logNamespaceVariables(namespace, sls, env);
    }
}
function logSlsVars(sls, env) {
    console.info('Variables found via Serverless Config:');
    console.info(JSON.stringify(sls, null, 2));
    const existingVars = Object.fromEntries(Object.keys(sls)
        .filter((k) => k in env)
        .map((k) => [k, env[k]]));
    console.info('Variables overriden by existing variables:');
    console.info(JSON.stringify(existingVars, null, 2));
}
function logAllVars(env) {
    console.info('All variables set for your child process');
    console.info(JSON.stringify(env, null, 2));
}
function logNamespaceVariables(namespace, sls, env) {
    const toLogAbout = Object.fromEntries(Object.entries(sls)
        .map(([k, v]) => [`${namespace}_${k}`, v])
        .filter(([k]) => !(k in env)));
    console.info('Namespaced Variables from Serverless:');
    console.info(JSON.stringify(toLogAbout, null, 2));
}
//# sourceMappingURL=runWithEnvVars.js.map